package com.bonus.material.basic.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.bonus.common.log.enums.OperaType;
import com.bonus.material.common.annotation.PreventRepeatSubmit;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.bonus.common.log.annotation.SysLog;
import com.bonus.common.security.annotation.RequiresPermissions;
import com.bonus.material.basic.domain.BmUnit;
import com.bonus.material.basic.service.IBmUnitService;
import com.bonus.common.core.web.controller.BaseController;
import com.bonus.common.core.web.domain.AjaxResult;
import com.bonus.common.core.utils.poi.ExcelUtil;
import com.bonus.common.core.web.page.TableDataInfo;

/**
 * 往来单位管理Controller
 * 
 * @author xsheng
 * @date 2024-09-27
 */
@Api(tags = "往来单位管理接口")
@RestController
@RequestMapping("/bm_unit")
public class BmUnitController extends BaseController
{
    @Autowired
    private IBmUnitService bmUnitService;

    /**
     * 查询往来单位管理列表
     */
    @ApiOperation(value = "查询往来单位管理列表")
    @RequiresPermissions("basic:unit:list")
    @GetMapping("/list")
    public TableDataInfo list(BmUnit bmUnit)
    {
        startPage();
        List<BmUnit> list = bmUnitService.selectBmUnitList(bmUnit);
        return getDataTable(list);
    }

    /**
     * 导出往来单位管理列表
     */
    @ApiOperation(value = "导出往来单位管理列表")
    @PreventRepeatSubmit
    @RequiresPermissions("basic:unit:export")
    @SysLog(title = "往来单位管理", businessType = OperaType.EXPORT, logType = 1,module = "仓储管理->导出往来单位管理")
    @PostMapping("/export")
    public void export(HttpServletResponse response, BmUnit bmUnit)
    {
        List<BmUnit> list = bmUnitService.selectBmUnitList(bmUnit);
        ExcelUtil<BmUnit> util = new ExcelUtil<BmUnit>(BmUnit.class);
        util.exportExcel(response, list, "往来单位管理数据");
    }

    /**
     * 获取往来单位管理详细信息
     */
    @ApiOperation(value = "获取往来单位管理详细信息")
    @RequiresPermissions("basic:unit:query")
    @GetMapping(value = "/{unitId}")
    public AjaxResult getInfo(@PathVariable("unitId") Long unitId)
    {
        return success(bmUnitService.selectBmUnitByUnitId(unitId));
    }

    /**
     * 新增往来单位管理
     */
    @ApiOperation(value = "新增往来单位管理")
    @PreventRepeatSubmit
    @RequiresPermissions("basic:unit:add")
    @SysLog(title = "往来单位管理", businessType = OperaType.INSERT, logType = 1,module = "仓储管理->新增往来单位管理")
    @PostMapping
    public AjaxResult add(@Validated @RequestBody BmUnit bmUnit)
    {
        return bmUnitService.insertBmUnit(bmUnit);
    }

    /**
     * 修改往来单位管理
     */
    @ApiOperation(value = "修改往来单位管理")
    @PreventRepeatSubmit
    @RequiresPermissions("basic:unit:edit")
    @SysLog(title = "往来单位管理", businessType = OperaType.UPDATE, logType = 1,module = "仓储管理->修改往来单位管理")
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody BmUnit bmUnit)
    {
        return bmUnitService.updateBmUnit(bmUnit);
    }

    /**
     * 删除往来单位管理
     */
    @ApiOperation(value = "删除往来单位管理")
    @PreventRepeatSubmit
    @RequiresPermissions("basic:unit:remove")
    @SysLog(title = "往来单位管理", businessType = OperaType.DELETE, logType = 1,module = "仓储管理->删除往来单位管理")
	@DeleteMapping("/{unitId}")
    public AjaxResult remove(@PathVariable("unitId") Long unitId)
    {
        return bmUnitService.deleteBmUnitByUnitId(unitId);
    }
}
